﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using System.Windows.Forms;
using HeuristicLab.Core.Networks;
using HeuristicLab.Visualization;

namespace HeuristicLab.Networks.Views.NetworkVisualization.Views {
  public class NetworkSelectChartMode : SelectChartMode {
    public NetworkSelectChartMode(ChartControl control) : base(control) { }

    public override void HandleOnKeyDown(object sender, KeyEventArgs e) {
      switch (e.KeyCode) {
        case Keys.Delete:
          if (e.KeyCode == Keys.Delete) {
            try {
              chartControl.SuspendRendering();
              var primitives = chartControl.Chart.Group.SelectedPrimitives.ToList();
              foreach (var connectionLine in primitives.OfType<ConnectionLine>()) {
                chartControl.Chart.Group.Remove(connectionLine);
                var startPortRectangle = connectionLine.StartPortRectangle;
                if (startPortRectangle != null) {
                  var startPort = (IMessagePort)startPortRectangle.NetworkItem;
                  if (startPort != null) startPort.ConnectedPort = null;
                }
              }

              foreach (var nodeRectangle in primitives.OfType<NodeRectangle>()) {
                chartControl.Chart.Group.Remove(nodeRectangle);
                var node = nodeRectangle.NetworkItem;
                if (node != null) {
                  var deletedPorts = node.Ports.OfType<IMessagePort>().ToList();
                  foreach (var port in deletedPorts)
                    port.ConnectedPort = null;

                  var connectionLines = chartControl.Chart.Group.Primitives.OfType<ConnectionLine>();
                  foreach (var connectionLine in connectionLines) {
                    var startPortRectangle = connectionLine.StartPortRectangle;
                    var endPortRectangle = connectionLine.EndPortRectangle;
                    if (startPortRectangle != null && endPortRectangle != null) {
                      var startPort = (IMessagePort)startPortRectangle.NetworkItem;
                      var endPort = (IMessagePort)endPortRectangle.NetworkItem;
                      if (startPort != null && endPort != null && deletedPorts.Contains(endPort))
                        startPort.ConnectedPort = null;
                    }
                  }

                  var parent = node.Parent;
                  if (parent != null)
                    parent.Nodes.Remove(node);
                }
              }
            } finally { chartControl.ResumeRendering(); }
          }
          break;
      }
    }

    public override void HandleOnMouseDoubleClick(object sender, MouseEventArgs e) {
      base.HandleOnMouseDoubleClick(sender, e);

      switch (e.Button) {
        case MouseButtons.Left:
          var p = chartControl.Chart.GetPrimitive(e.Location) as NodeRectangle;
          if (p != null)
            MainForm.MainFormManager.MainForm.ShowContent(p.NetworkItem);
          break;
      }
    }
  }
}
